function[beta]=cal_polyn_im2gl(pl_1,pl_2,im_pl_1,im_pl_2,c1pixnum,method,n);

% function to create coefficients for nth order polynomial calibration Taylor series expansion
% requires: calibration points in 3D coords, and image coords for each camera
% the output function can be used to convert from image to global coordinates
%
%Inputs: pl_1    - x,y,z global coordinates of calibration plate 1
%        pl_2    - x,y,z global coordinates of calibration plate 2
%        im_pl_1 - x,y   image coordinates of calibration plate 1
%        im_pl_2 - x,y   image coordinates of calibration plate 2
%        c1pixnum- resolution of image 1
%        method  - least sqaures method (1=ordinary, 2=robust)
%        n       - order of polynomial (will accept 1st to 5th order)
%Outputs: beta   - array of polynominal coefficients for both plates and coordinates
%
%
%
%
% created by: Nick Worth 10/4/08

if nargin==5
   method=2; % if not specified then use robust method
   n=3;      % and if not specified use 3rd order
elseif nargin==6
   n=3;      % and if not specified use 3rd order
end

% check inputs are in correct format (ie/ columns of data, not rows)
if size(pl_1,2)>size(pl_1,1) | size(pl_2,2)>size(pl_2,1) | size(im_pl_1,2)>size(im_pl_1,1) | size(im_pl_2,2)>size(im_pl_2,1)
   disp('WARNING - need to makes sure inputs are in columns not rows of data')
end

% FRONT X coord
F=pl_1(:,1);
x=[im_pl_1(:,1)];         y=[im_pl_1(:,2)];
xma=x-c1pixnum(2)/2; ymb=y-c1pixnum(1)/2;

%create the design matrix - based on the specified polynomial
if n==1
X=[ones(size(y)),xma,ymb];%for n=1
elseif n==2
X=[ones(size(y)),xma,ymb,((1/factorial(2))*(xma.^2)),((2/factorial(2))*(xma.*ymb)),((1/factorial(2))*(ymb.^2))];%for n=2
elseif n==3
X=[ones(size(y)),xma,ymb,((1/factorial(2))*(xma.^2)),((2/factorial(2))*(xma.*ymb)),((1/factorial(2))*(ymb.^2)),((1/factorial(3))*(xma.^3)),((3/factorial(3))*((xma.^2).*ymb)),((3/factorial(3))*((ymb.^2).*xma)),((1/factorial(3))*(ymb.^3))]; %for n=3
elseif n==4
X=[ones(size(y)),xma,ymb,((1/factorial(2))*(xma.^2)),((2/factorial(2))*(xma.*ymb)),((1/factorial(2))*(ymb.^2)),((1/factorial(3))*(xma.^3)),((3/factorial(3))*((xma.^2).*ymb)),((3/factorial(3))*((ymb.^2).*xma)),((1/factorial(3))*(ymb.^3)),((1/factorial(4))*(xma.^4)),((4/factorial(4))*((xma.^3).*ymb)),((6/factorial(4))*((xma.^2).*(ymb.^2))),((4/factorial(4))*(xma.*(ymb.^3))),((1/factorial(4))*(ymb.^4))]; %for n=4
elseif n==5
X=[ones(size(y)),xma,ymb,((1/factorial(2))*(xma.^2)),((2/factorial(2))*(xma.*ymb)),((1/factorial(2))*(ymb.^2)),((1/factorial(3))*(xma.^3)),((3/factorial(3))*((xma.^2).*ymb)),((3/factorial(3))*((ymb.^2).*xma)),((1/factorial(3))*(ymb.^3)),((1/factorial(4))*(xma.^4)),((4/factorial(4))*((xma.^3).*ymb)),((6/factorial(4))*((xma.^2).*(ymb.^2))),((4/factorial(4))*(xma.*(ymb.^3))),((1/factorial(4))*(ymb.^4)),((1/factorial(5))*(xma.^5)),((5/factorial(5))*((xma.^4).*ymb)),((10/factorial(5))*((xma.^3).*(ymb.^2))),((10/factorial(5))*((xma.^2).*(ymb.^3))),((5/factorial(5))*(xma.*(ymb.^4))),((1/factorial(5))*(ymb.^5))]; %for n=5
else
disp('Warning polynomials of order n>5 are currently not supported')
end
if method==1
beta_FX=X\F; %use matlabs linear Least Squares function to find the coefficients beta_1 and beta_2
elseif method==2
beta_FX=robustfit(X(:,2:end),F); %use robust estimation method
end


% FRONT Y coord
F=pl_1(:,2);
x=[im_pl_1(:,1)];        ; y=[im_pl_1(:,2)];
xma=x-c1pixnum(2)/2; ymb=y-c1pixnum(1)/2;

%create the design matrix - based on the specified polynomial
if n==1
X=[ones(size(y)),xma,ymb];%for n=1
elseif n==2
X=[ones(size(y)),xma,ymb,((1/factorial(2))*(xma.^2)),((2/factorial(2))*(xma.*ymb)),((1/factorial(2))*(ymb.^2))];%for n=2
elseif n==3
X=[ones(size(y)),xma,ymb,((1/factorial(2))*(xma.^2)),((2/factorial(2))*(xma.*ymb)),((1/factorial(2))*(ymb.^2)),((1/factorial(3))*(xma.^3)),((3/factorial(3))*((xma.^2).*ymb)),((3/factorial(3))*((ymb.^2).*xma)),((1/factorial(3))*(ymb.^3))]; %for n=3
elseif n==4
X=[ones(size(y)),xma,ymb,((1/factorial(2))*(xma.^2)),((2/factorial(2))*(xma.*ymb)),((1/factorial(2))*(ymb.^2)),((1/factorial(3))*(xma.^3)),((3/factorial(3))*((xma.^2).*ymb)),((3/factorial(3))*((ymb.^2).*xma)),((1/factorial(3))*(ymb.^3)),((1/factorial(4))*(xma.^4)),((4/factorial(4))*((xma.^3).*ymb)),((6/factorial(4))*((xma.^2).*(ymb.^2))),((4/factorial(4))*(xma.*(ymb.^3))),((1/factorial(4))*(ymb.^4))]; %for n=4
elseif n==5
X=[ones(size(y)),xma,ymb,((1/factorial(2))*(xma.^2)),((2/factorial(2))*(xma.*ymb)),((1/factorial(2))*(ymb.^2)),((1/factorial(3))*(xma.^3)),((3/factorial(3))*((xma.^2).*ymb)),((3/factorial(3))*((ymb.^2).*xma)),((1/factorial(3))*(ymb.^3)),((1/factorial(4))*(xma.^4)),((4/factorial(4))*((xma.^3).*ymb)),((6/factorial(4))*((xma.^2).*(ymb.^2))),((4/factorial(4))*(xma.*(ymb.^3))),((1/factorial(4))*(ymb.^4)),((1/factorial(5))*(xma.^5)),((5/factorial(5))*((xma.^4).*ymb)),((10/factorial(5))*((xma.^3).*(ymb.^2))),((10/factorial(5))*((xma.^2).*(ymb.^3))),((5/factorial(5))*(xma.*(ymb.^4))),((1/factorial(5))*(ymb.^5))]; %for n=5
else
disp('Warning polynomials of order n>5 are currently not supported')
end
%now use matlabs linear Least Squares function to find the coefficients beta_1 and beta_2
if method==1
beta_FY=X\F; %use matlabs linear Least Squares function to find the coefficients beta_1 and beta_2
elseif method==2
beta_FY=robustfit(X(:,2:end),F); %use robust estimation method
end


% BACK X coord
F=pl_2(:,1);
x=[im_pl_2(:,1)];      y=[im_pl_2(:,2)];
xma=x-c1pixnum(2)/2; ymb=y-c1pixnum(1)/2;

%create the design matrix - based on the specified polynomial
if n==1
X=[ones(size(y)),xma,ymb];%for n=1
elseif n==2
X=[ones(size(y)),xma,ymb,((1/factorial(2))*(xma.^2)),((2/factorial(2))*(xma.*ymb)),((1/factorial(2))*(ymb.^2))];%for n=2
elseif n==3
X=[ones(size(y)),xma,ymb,((1/factorial(2))*(xma.^2)),((2/factorial(2))*(xma.*ymb)),((1/factorial(2))*(ymb.^2)),((1/factorial(3))*(xma.^3)),((3/factorial(3))*((xma.^2).*ymb)),((3/factorial(3))*((ymb.^2).*xma)),((1/factorial(3))*(ymb.^3))]; %for n=3
elseif n==4
X=[ones(size(y)),xma,ymb,((1/factorial(2))*(xma.^2)),((2/factorial(2))*(xma.*ymb)),((1/factorial(2))*(ymb.^2)),((1/factorial(3))*(xma.^3)),((3/factorial(3))*((xma.^2).*ymb)),((3/factorial(3))*((ymb.^2).*xma)),((1/factorial(3))*(ymb.^3)),((1/factorial(4))*(xma.^4)),((4/factorial(4))*((xma.^3).*ymb)),((6/factorial(4))*((xma.^2).*(ymb.^2))),((4/factorial(4))*(xma.*(ymb.^3))),((1/factorial(4))*(ymb.^4))]; %for n=4
elseif n==5
X=[ones(size(y)),xma,ymb,((1/factorial(2))*(xma.^2)),((2/factorial(2))*(xma.*ymb)),((1/factorial(2))*(ymb.^2)),((1/factorial(3))*(xma.^3)),((3/factorial(3))*((xma.^2).*ymb)),((3/factorial(3))*((ymb.^2).*xma)),((1/factorial(3))*(ymb.^3)),((1/factorial(4))*(xma.^4)),((4/factorial(4))*((xma.^3).*ymb)),((6/factorial(4))*((xma.^2).*(ymb.^2))),((4/factorial(4))*(xma.*(ymb.^3))),((1/factorial(4))*(ymb.^4)),((1/factorial(5))*(xma.^5)),((5/factorial(5))*((xma.^4).*ymb)),((10/factorial(5))*((xma.^3).*(ymb.^2))),((10/factorial(5))*((xma.^2).*(ymb.^3))),((5/factorial(5))*(xma.*(ymb.^4))),((1/factorial(5))*(ymb.^5))]; %for n=5
else
disp('Warning polynomials of order n>5 are currently not supported')
end
%now use matlabs linear Least Squares function to find the coefficients beta_1 and beta_2
if method==1
beta_BX=X\F; %use matlabs linear Least Squares function to find the coefficients beta_1 and beta_2
elseif method==2
beta_BX=robustfit(X(:,2:end),F); %use robust estimation method
end

% BACK Y coord
F=pl_2(:,2);
x=[im_pl_2(:,1)];       ; y=[im_pl_2(:,2)];
xma=x-c1pixnum(2)/2; ymb=y-c1pixnum(1)/2;

%create the design matrix - based on the specified polynomial
if n==1
X=[ones(size(y)),xma,ymb];%for n=1
elseif n==2
X=[ones(size(y)),xma,ymb,((1/factorial(2))*(xma.^2)),((2/factorial(2))*(xma.*ymb)),((1/factorial(2))*(ymb.^2))];%for n=2
elseif n==3
X=[ones(size(y)),xma,ymb,((1/factorial(2))*(xma.^2)),((2/factorial(2))*(xma.*ymb)),((1/factorial(2))*(ymb.^2)),((1/factorial(3))*(xma.^3)),((3/factorial(3))*((xma.^2).*ymb)),((3/factorial(3))*((ymb.^2).*xma)),((1/factorial(3))*(ymb.^3))]; %for n=3
elseif n==4
X=[ones(size(y)),xma,ymb,((1/factorial(2))*(xma.^2)),((2/factorial(2))*(xma.*ymb)),((1/factorial(2))*(ymb.^2)),((1/factorial(3))*(xma.^3)),((3/factorial(3))*((xma.^2).*ymb)),((3/factorial(3))*((ymb.^2).*xma)),((1/factorial(3))*(ymb.^3)),((1/factorial(4))*(xma.^4)),((4/factorial(4))*((xma.^3).*ymb)),((6/factorial(4))*((xma.^2).*(ymb.^2))),((4/factorial(4))*(xma.*(ymb.^3))),((1/factorial(4))*(ymb.^4))]; %for n=4
elseif n==5
X=[ones(size(y)),xma,ymb,((1/factorial(2))*(xma.^2)),((2/factorial(2))*(xma.*ymb)),((1/factorial(2))*(ymb.^2)),((1/factorial(3))*(xma.^3)),((3/factorial(3))*((xma.^2).*ymb)),((3/factorial(3))*((ymb.^2).*xma)),((1/factorial(3))*(ymb.^3)),((1/factorial(4))*(xma.^4)),((4/factorial(4))*((xma.^3).*ymb)),((6/factorial(4))*((xma.^2).*(ymb.^2))),((4/factorial(4))*(xma.*(ymb.^3))),((1/factorial(4))*(ymb.^4)),((1/factorial(5))*(xma.^5)),((5/factorial(5))*((xma.^4).*ymb)),((10/factorial(5))*((xma.^3).*(ymb.^2))),((10/factorial(5))*((xma.^2).*(ymb.^3))),((5/factorial(5))*(xma.*(ymb.^4))),((1/factorial(5))*(ymb.^5))]; %for n=5
else
disp('Warning polynomials of order n>5 are currently not supported')
end
%now use matlabs linear Least Squares function to find the coefficients beta_1 and beta_2
if method==1
beta_BY=X\F; %use matlabs linear Least Squares function to find the coefficients beta_1 and beta_2
elseif method==2
beta_BY=robustfit(X(:,2:end),F); %use robust estimation method
end

beta=[beta_FX, beta_FY, beta_BX, beta_BY];