function []=find_calpoints_tomojet_May2017(find_phi)

% function to find the calibration points on each image, for 2 planes of points
% and calculate the pixel coordinates of these, to sub pixel accuracy
% note, can use the program 'make_fake_cal.m' to make fake images if you don't have any

% this program has been modified to include a more rigourous definition of
% the coordinate system, and allow easier offsets for the calibration
% planes from each other.

% NEW VERSION FOR FLAME TOMOGRAPHY:
% The second version for flames, calibrates only 1 image pair, but also
% includes advanced features for defining shifts based on the refractive
% index of different regions in the image.

% THIS VERSION IS SUITABLE FOR UNCONFINED GEOMETRY

global c1pixnum
global c1_beta_im2gl pl_1_o pl_2_o pl_1 pl_2

if nargin==0
    find_phi=0;
end


% DIRECTORY------------------------------------------------------------
%project_dir='/Users/nickworth/Documents/FORTRAN/29apr2017_4cam_testrun';
project_dir='C:\Users\Erik\Skole\10.Semester\Calibration code';
calibration_save_dir='CalibrationData';

dir=project_dir;
save_dir=[project_dir,'/',calibration_save_dir];

cd(dir)


cams=1:1;
output_ref_relation=0;
outputfac=1;

% CONTROLS-------------------------------------------------------------
wanna_output=2; % do you want to display the calibration output (for use in fortran)? 1=old style; 2= save to file for fortran to read
wanna_remap=1;  % do you want to display the first test - remapping pixel points back into world coords?
wanna_remap2=1; % do you want to display the second test - numbered points on image planes?
wanna_remap3=1; % do you want to display the calibration mapped image coords on the figure 2
wanna_error=1;  %do you want to display the calibration errors?
cal_plate_inspect=0; %do you want to inspect the calibration plate?
confirm_coord_matching=0; %do you want to check which points are mapping to which from the image domain onto the image domain?

wanna_preprocess=1; % do you want to whack through the bandpass image preprocessor? - this is always recommended
preprocess_part_diam=25; %if preprocessing size of particles
preprocess_noise=1; %if preprocessing, size of noise
filtermin=10; %intensity to remove

wanna_zoom_in=1; %if the calibration plate is hard to see, zoom in first and then pic the points
zoom_size=800; %size of zoomed in window in pixels

method=2; %select method for point estimation. 1= simple 2 line estimate of point spread, 2=full LS method
LSm_pf=1; %select method for least squares estimation during point finding stages (1=ordinary, 2=robust)
LSm_c=2; % select method for least sqaures estimation during calibration stage (1=ordinary, 2=robust)
fort_lim=-1; %limit the accuracy of the fortran output, see the accuracy check to ensure this is ok
poly_n=3;

wanna_noRIC=0;    %creates version with refractive correction taken into account
wanna_overwrite=1;%overwrites standard calibration with the refractive version

wanna_save_thesis_im=0;

for current_ref=cams

    % SET CURRENT CALIBRATION ANGLE ---------------------------------------
    % current calibration plates. ie/ c*_cal_f, where *=current_ref
    output_ref=((current_ref+output_ref_relation)/outputfac);  % define output number for this plate, ie/ c*be, where *=output_ref


    % Enter calibration image names


    
    filetype='tif';

    % Peak finding paramaters
    boxs=45; %size of box used around estimated center of calibration point in pixels, ie how big are the dots on each image

    %choose variable type - or one of fixed calc plate types -
    cal_plate_selection=2;


    if cal_plate_selection==1 % plates traversed 5mm forward and back
        name1f=['','C00',num2str(current_ref),'H001S0001/','C00',num2str(current_ref),'H001S0001000001'];
        name1b=['','C00',num2str(current_ref),'H001S0001/','C00',num2str(current_ref),'H001S0001000001'];

        
        correct_method=2;
        mis(1,1:2)=[0,0,];
        mis(2,1:2)=[0,0,];
        mis(3,1:2)=[0,0,];
        mis(4,1:2)=[0,0,];
        mis_pm=[10,10]; %distance of misalignment coords from center point in dots
        pl_1_o=[0,0,-5]; pl_2_o=[0,0,5]; %origins global coords (x,y,z) (mm)
        pl_num=[12,12]; % number of calibration points [x,y] - should always be odd and doesn't matter if its too big
        pl_1_d=[5,5]; pl_2_d=[5,5]; % [x,y] spacing

        %calculate misalignment parameters
        pl1_4pts=[pl_1_o(1)-(mis_pm(1)*pl_1_d(1)),pl_1_o(2)+(mis_pm(1)*pl_1_d(1)); pl_1_o(1)+(mis_pm(1)*pl_1_d(1)),pl_1_o(2)+(mis_pm(1)*pl_1_d(1));pl_1_o(1)+(mis_pm(1)*pl_1_d(1)),pl_1_o(2)-(mis_pm(1)*pl_1_d(1));pl_1_o(1)-(mis_pm(1)*pl_1_d(1)),pl_1_o(2)-(mis_pm(1)*pl_1_d(1))]; pl2_4pts=pl1_4pts+mis;
        pt1_mis=mis(1,1:2);% [-pl_2_d(1)/2,pl_2_d(2)/2] %calculate difference between central alignment and actual alignment of point 1

        yvar=(((pl2_4pts(2,2)-pl2_4pts(1,2))+(pl2_4pts(3,2)-pl2_4pts(4,2)))/2)/(2*mis_pm(1)*pl_2_d(1)); %mean variation of y per mm, wrt plane 1 from reference point -3*x spacing from center %calculate simple linear variation of plane 2 from plane 1, as a function of x and y position
        xvar=(((pl2_4pts(4,1)-pl2_4pts(1,1))+(pl2_4pts(3,1)-pl2_4pts(2,1)))/2)/(2*mis_pm(2)*pl_2_d(2)); %mean variation of x per mm, wrt plane 1 from reference point -3*y spacing from center
    elseif cal_plate_selection==2 % plate in centre

        name1f=['','','','','C00',num2str(current_ref),'H001S0001000002'];
        name1b=['','','','','C00',num2str(current_ref),'H001S0001000002'];

        correct_method=2;
        mis(1,1:2)=[-2.5,-2.5,];
        mis(2,1:2)=[-2.5,-2.5,];
        mis(3,1:2)=[-2.5,-2.5,];
        mis(4,1:2)=[-2.5,-2.5,];
        mis_pm=[10,10]; %distance of misalignment coords from center point in dots
        pl_1_o=[0,0,-1]; pl_2_o=[-15,-15,0]; %origins global coords (x,y,z) (mm)
        pl_num=[13,13]; % number of calibration points [x,y] - should always be odd and doesn't matter if its too big
        pl_1_d=[5,5]; pl_2_d=[5,5]; % [x,y] spacing

        %calculate misalignment parameters
        pl1_4pts=[pl_1_o(1)-(mis_pm(1)*pl_1_d(1)),pl_1_o(2)+(mis_pm(1)*pl_1_d(1)); pl_1_o(1)+(mis_pm(1)*pl_1_d(1)),pl_1_o(2)+(mis_pm(1)*pl_1_d(1));pl_1_o(1)+(mis_pm(1)*pl_1_d(1)),pl_1_o(2)-(mis_pm(1)*pl_1_d(1));pl_1_o(1)-(mis_pm(1)*pl_1_d(1)),pl_1_o(2)-(mis_pm(1)*pl_1_d(1))]; pl2_4pts=pl1_4pts+mis;
        pt1_mis=mis(1,1:2);% [-pl_2_d(1)/2,pl_2_d(2)/2] %calculate difference between central alignment and actual alignment of point 1

        yvar=(((pl2_4pts(2,2)-pl2_4pts(1,2))+(pl2_4pts(3,2)-pl2_4pts(4,2)))/2)/(2*mis_pm(1)*pl_2_d(1)); %mean variation of y per mm, wrt plane 1 from reference point -3*x spacing from center %calculate simple linear variation of plane 2 from plane 1, as a function of x and y position
        xvar=(((pl2_4pts(4,1)-pl2_4pts(1,1))+(pl2_4pts(3,1)-pl2_4pts(2,1)))/2)/(2*mis_pm(2)*pl_2_d(2)); %mean variation of x per mm, wrt plane 1 from reference point -3*y spacing from center
    
    end





    %first define the calibration planes in global coordinates based on a left
    %handed coordinate system, with references to these through a simple
    %"distance from the central point" type scheme

    pl_1_cen=ceil(pl_num/2);
    pl_2_cen=ceil(pl_num/2);

    for xpos=1:(pl_num(1))
        for ypos=1:(pl_num(2))
            pl_1_x_ref(ypos,xpos)=xpos-pl_1_cen(1);
            pl_1_y_ref(ypos,xpos)=ypos-pl_1_cen(2);
        end
    end


    pl_1_x=pl_1_x_ref*pl_1_d(1);
    pl_1_y=pl_1_y_ref*pl_1_d(2);
    pl_1_z=ones(size(pl_1_y))*pl_1_o(3);



    for xpos=1:(pl_num(1))
        for ypos=1:(pl_num(2))
            pl_2_x(ypos,xpos)=pl_1_x(ypos,xpos)+pt1_mis(1);
            pl_2_y(ypos,xpos)=pl_1_y(ypos,xpos)+pt1_mis(2);
            ref_dist_x=pl2_4pts(1,1)-pl_2_x(ypos,xpos);
            ref_dist_y=pl2_4pts(1,2)-pl_2_y(ypos,xpos);
            loc_adj(2)=ref_dist_x*yvar;
            loc_adj(1)=ref_dist_y*xvar;
            %calc local adjustment
            pl_2_x(ypos,xpos)=pl_2_x(ypos,xpos)-loc_adj(1);  %adjust with respect to plane 1 point
            pl_2_y(ypos,xpos)=pl_2_y(ypos,xpos)-loc_adj(2);
        end
    end
    



    pl_2_z=ones(size(pl_2_y))*pl_2_o(3);

    %define gamma coords from these arrays - remember ref(y,x)
    pl1_phi=[pl_1_x(pl_1_cen(2),pl_1_cen(1)),pl_1_y(pl_1_cen(2),pl_1_cen(1)),pl_1_z(pl_1_cen(2),pl_1_cen(1));
        pl_1_x(pl_1_cen(2),pl_1_cen(1)+1),pl_1_y(pl_1_cen(2),pl_1_cen(1)+1),pl_1_z(pl_1_cen(2),pl_1_cen(1)+1);
        pl_1_x(pl_1_cen(2)-1,pl_1_cen(1)),pl_1_y(pl_1_cen(2)-1,pl_1_cen(1)),pl_1_z(pl_1_cen(2)-1,pl_1_cen(1))];
    pl2_phi=[pl_2_x(pl_2_cen(2),pl_2_cen(1)),pl_2_y(pl_1_cen(2),pl_2_cen(1)),pl_2_z(pl_2_cen(2),pl_2_cen(1));
        pl_2_x(pl_2_cen(2),pl_2_cen(1)+1),pl_2_y(pl_1_cen(2),pl_2_cen(1)+1),pl_2_z(pl_2_cen(2),pl_2_cen(1)+1);
        pl_2_x(pl_2_cen(2)-1,pl_2_cen(1)),pl_2_y(pl_1_cen(2)-1,pl_2_cen(1)),pl_2_z(pl_2_cen(2)-1,pl_2_cen(1))];


    
    %plot check the calibration plate misalignment to see if it looks like it should
    if cal_plate_inspect==1
        figure(9), clf, hold on
        for xpos=1:(pl_num(1))
            for ypos=1:(pl_num(2))
                plot(pl_1_x(ypos,xpos),pl_1_y(ypos,xpos),'rx')
                plot(pl_2_x(ypos,xpos),pl_2_y(ypos,xpos),'gx')
            end
        end
        plot(pl1_phi(:,1),pl1_phi(:,2),'ro')
        plot(pl2_phi(:,1),pl2_phi(:,2),'go')
        xlabel('x'), ylabel('y')
        title('Calibration plate inspection:: red = plane 1, green = plane 2, both mapped onto same plane for viewing ease')
        %annotate calibration plate graph to include data
        plot([pl1_4pts(1,1),pl2_4pts(1,1)],[pl1_4pts(1,2),pl2_4pts(1,2)],'b')
        plot([pl1_4pts(2,1),pl2_4pts(2,1)],[pl1_4pts(2,2),pl2_4pts(2,2)],'b')
        plot([pl1_4pts(3,1),pl2_4pts(3,1)],[pl1_4pts(3,2),pl2_4pts(3,2)],'b')
        plot([pl1_4pts(4,1),pl2_4pts(4,1)],[pl1_4pts(4,2),pl2_4pts(4,2)],'b')
        pause
    end

    % clear the figures
    figure(1), clf, figure(2), clf
    % Load Calibration images

    im1f=double(imread(name1f,filetype));
    im1b=double(imread(name1b,filetype));

    
% 
%     figure(3), clf,
%     imagesc(im1f),
%     colormap('gray')
%     daspect([1 1 1])
%     axis off
%     whitebg('w')
%     set(gcf,'InvertHardcopy','on')
%     caxis([0 800])
%     cd('/Users/nick/Documents/current files/191010 Twin flame tomo unconfined paper/figs')
%     imagesave_RA('cal_plate_front',0,4,3)
%     
%     figure(3), clf,
%     imagesc(im1b),
%     colormap('gray')
%     daspect([1 1 1])
%     axis off
%     whitebg('w')
%     set(gcf,'InvertHardcopy','on')
%     caxis([0 800])
%     cd('/Users/nick/Documents/current files/191010 Twin flame tomo unconfined paper/figs')
%     imagesave_RA('cal_plate_back',0,4,3)

    
    
    % temporary code to place marker dot where dots fell off!
    % if current_ref==2
    % im1f(600:605,440:445)=500;
    % im1b(595:600,610:615)=500;
    % elseif current_ref==3
    % im1f(600:605,585:590)=500;
    % im1b(600:605,435:440)=500;
    % elseif current_ref==4
    % im1f(585:590,585:590)=500;
    % im1b(585:590,320:325)=500;
    % end






    if wanna_preprocess==1
        %normalise intensity in range 0 to 255
        im1f=(im1f/max(max(im1f)))*255;
        im1b=(im1b/max(max(im1b)))*255;

        im1fold=im1f; 
        im1bold=im1b;
        im1f=bpass(im1f,preprocess_noise,preprocess_part_diam);
        im1b=bpass(im1b,preprocess_noise,preprocess_part_diam);

        % add surrounding section
%         paddnum=100;
%         backgroundlevel=median(median(im1f)),;
%         blank1=ones(size(im1f)+2*[paddnum paddnum]);
%         blank1(paddnum:paddnum+size(im1f,1)-1,paddnum:paddnum+size(im1f,2)-1)=im1f;
%         im1f=blank1;
        
        %normalise intensity in range 0 to 255
        im1f=(im1f/max(max(im1f)))*255;
        im1b=(im1b/max(max(im1b)))*255;

        %remove background noise
        im1f(im1f(:,:)<filtermin)=0;
        im1b(im1b(:,:)<filtermin)=0;

        
% saturate
% saturation_threshold=0.80;
% im1f(im1f>(saturation_threshold*max(max(im1f))))=saturation_threshold*max(max(im1f));
% im1b(im1b>(saturation_threshold*max(max(im1b))))=saturation_threshold*max(max(im1b));

        
        
        figure(5), clf, colormap('jet')
        title('Comparison of preprocessed')
        subplot(2,2,1),imagesc(im1fold);subplot(2,2,2),imagesc(im1f);
        subplot(2,2,3),imagesc(im1bold);subplot(2,2,4),imagesc(im1b);
           
        
    end


    % ken the size of the images, and put into the classic pixnum format
    c1pixnum=size(im1f);

    % display the calibration images
    figure(2), colormap('jet')
    subplot(1,2,1), imagesc(im1f);
    subplot(1,2,2), imagesc(im1b);

    % display the calibration images
    %figure(1), colormap('gray'),
    %subplot(2,2,1), imagesc(im1); subplot(2,2,2), imagesc(im2); subplot(2,2,3), imagesc(im3), if cams>3, subplot(2,2,4), imagesc(im4), end

    % indentify location of 3 key points in Gamma shape pattern in center of cal plate
    % this is only to the nearest mm
    % (at first through manual inspection and entering the coords, but later by clicking on the image)
    % current convention:- for odd numbers (center point),
    %                      for even numbers (left and top of center four points)


    if find_phi==0

        winlim=5;
        figure(1), colormap('jet'), subplot(2,1,1), imagesc(im1f);
        if wanna_zoom_in==1,
            title('Camera 1: Please choose zoom center')
            [zoomcenx,zoomceny]=ginput(1);
            xlim([zoomcenx-(zoom_size/2) zoomcenx+(zoom_size/2)]),   ylim([zoomceny-(zoom_size/2) zoomceny+(zoom_size/2)])
        end
        title('Camera 1: Please choose 3 gamma points for plane 1')
        subplot(2,1,1), [c1_pl1x(1,1),c1_pl1y(1,1)]=ginput(1); hold on, plot(c1_pl1x(1,1),c1_pl1y(1,1),'go'),
        subplot(2,1,2), cla, imagesc(im1f); hold on, xlim([(c1_pl1x(1,1)-winlim) (c1_pl1x(1,1)+winlim)]),ylim([(c1_pl1y(1,1)-winlim) (c1_pl1y(1,1)+winlim)]),plot(c1_pl1x(1,1),c1_pl1y(1,1),'go')
        subplot(2,1,1), [c1_pl1x(2,1),c1_pl1y(2,1)]=ginput(1); plot(c1_pl1x(2,1),c1_pl1y(2,1),'go')
        subplot(2,1,2), cla, imagesc(im1f); hold on, xlim([(c1_pl1x(2,1)-winlim) (c1_pl1x(2,1)+winlim)]),ylim([(c1_pl1y(2,1)-winlim) (c1_pl1y(2,1)+winlim)]),plot(c1_pl1x(2,1),c1_pl1y(2,1),'go')
        subplot(2,1,1),[c1_pl1x(3,1),c1_pl1y(3,1)]=ginput(1); plot(c1_pl1x(3,1),c1_pl1y(3,1),'go')
        subplot(2,1,2), cla, imagesc(im1f); hold on, xlim([(c1_pl1x(3,1)-winlim) (c1_pl1x(3,1)+winlim)]),ylim([(c1_pl1y(3,1)-winlim) (c1_pl1y(3,1)+winlim)]),plot(c1_pl1x(3,1),c1_pl1y(3,1),'go')


        figure(1), clf, colormap('jet'), subplot(2,1,1), imagesc(im1b);

        if wanna_zoom_in==1,
            title('Camera 1: Please choose zoom center')
            [zoomcenx,zoomceny]=ginput(1);
            xlim([zoomcenx-(zoom_size/2) zoomcenx+(zoom_size/2)]),   ylim([zoomceny-(zoom_size/2) zoomceny+(zoom_size/2)])
        end

        subplot(2,1,1), title('Camera 1: Please choose 3 gamma points for plane 2')
        subplot(2,1,1),[c1_pl2x(1,1),c1_pl2y(1,1)]=ginput(1);  hold on, plot(c1_pl2x(1,1),c1_pl2y(1,1),'ro')
        subplot(2,1,2), cla, imagesc(im1b); hold on, xlim([(c1_pl2x(1,1)-winlim) (c1_pl2x(1,1)+winlim)]),ylim([(c1_pl2y(1,1)-winlim) (c1_pl2y(1,1)+winlim)]),plot(c1_pl2x(1,1),c1_pl2y(1,1),'ro')
        subplot(2,1,1),[c1_pl2x(2,1),c1_pl2y(2,1)]=ginput(1); plot(c1_pl2x(2,1),c1_pl2y(2,1),'ro')
        subplot(2,1,2), cla, imagesc(im1b); hold on, xlim([(c1_pl2x(2,1)-winlim) (c1_pl2x(2,1)+winlim)]),ylim([(c1_pl2y(2,1)-winlim) (c1_pl2y(2,1)+winlim)]),plot(c1_pl2x(2,1),c1_pl2y(2,1),'ro')
        subplot(2,1,1),[c1_pl2x(3,1),c1_pl2y(3,1)]=ginput(1); plot(c1_pl2x(3,1),c1_pl2y(3,1),'ro')
        subplot(2,1,2), cla, imagesc(im1b); hold on, xlim([(c1_pl2x(3,1)-winlim) (c1_pl2x(3,1)+winlim)]),ylim([(c1_pl2y(3,1)-winlim) (c1_pl2y(3,1)+winlim)]),plot(c1_pl2x(3,1),c1_pl2y(3,1),'ro')

        %collate these
        c1_cp=[c1_pl1x,c1_pl1y;c1_pl2x,c1_pl2y];

        cd(save_dir)
        save(['gamma_coords_',num2str(current_ref)],'c1_cp')
        cd(dir)
        
    elseif find_phi==1;
        cd(save_dir)
        load(['gamma_coords_',num2str(current_ref)])
        cd(dir)
    end

    


    figure(1), clf, colormap('jet'),
    subplot(1,2,1), imagesc(im1f);
    subplot(1,2,2), imagesc(im1b);


    % initialise the counter for the number of double peak points
    dubpco=0;



    % execute point location program for each camera
    if method==1
        %[c1pl1_fi,c1pl2_fi,dubpco,c1pl1_inr,c1pl2_inr,c1cp_fi]=point_locator(c1_cp,im1,c1pixnum,pl_1_num,pl_2_num,boxs,1,dubpco,pl1_phi,pl2_phi,pl_1_d,pl_2_d,pl_1_o,pl_2_o,LSm_pf);
        %[c2pl1_fi,c2pl2_fi,dubpco,c2pl1_inr,c2pl2_inr,c2cp_fi]=point_locator(c2_cp,im2,c2pixnum,pl_1_num,pl_2_num,boxs,2,dubpco,pl1_phi,pl2_phi,pl_1_d,pl_2_d,pl_1_o,pl_2_o,LSm_pf);
        %[c3pl1_fi,c3pl2_fi,dubpco,c3pl1_inr,c3pl2_inr,c3cp_fi]=point_locator(c3_cp,im3,c3pixnum,pl_1_num,pl_2_num,boxs,3,dubpco,pl1_phi,pl2_phi,pl_1_d,pl_2_d,pl_1_o,pl_2_o,LSm_pf);
        %if cams>3, [c4pl1_fi,c4pl2_fi,dubpco,c4pl1_inr,c4pl2_inr,c4cp_fi]=point_locator(c4_cp,im4,c4pixnum,pl_1_num,pl_2_num,boxs,4,dubpco,pl1_phi,pl2_phi,pl_1_d,pl_2_d,pl_1_o,pl_2_o,LSm_pf); end
        disp('warning:: this method has not been updated')
    elseif method==2
        [c1pl1_fi,c1pl2_fi,dubpco,c1pl_1,c1pl_2,c1cp_fi]=point_locator2(c1_cp,im1f,im1b,c1pixnum,pl_num,boxs,1,dubpco,pl1_phi,pl2_phi,pl_1_d,pl_2_d,pl_1_o,pl_2_o,LSm_pf,correct_method,pl2_4pts,yvar,xvar,pl_1_x,pl_1_y,pl_2_x,pl_2_y,pl_1_cen,pl_2_cen,confirm_coord_matching,poly_n);

    end



    %report on the number of double peak points
    if dubpco>0
        disp(['WARNING ',num2str(dubpco),' points have more than 1 peak'])
        disp('currently the first is assigned abritarily')
    end


    if method==1
        %now eliminate the points which are out of range in both the global and image coordinates
        c1pl_1=pl_1(c1pl1_inr>0,1:3); c1pl1_fi=c1pl1_fi(c1pl1_inr>0,1:2);
        c1pl_2=pl_2(c1pl2_inr>0,1:3); c1pl2_fi=c1pl2_fi(c1pl2_inr>0,1:2);
    end

    % checker for order of points and locations in the global and image domains
    checker=0;
    if checker==1;
        figure(7), clf, subplot(1,2,2), imagesc(im2)

        for i=1:length(c2pl_1),
            subplot(1,2,1), set(gca,'YDir','reverse')
            subplot(1,2,1),hold on, plot(pl1_phi(1,1),pl1_phi(1,2),'b*'), plot3(c2pl_1(i,1),c2pl_1(i,2),c2pl_1(i,3),'ro'),
            subplot(1,2,2),hold on,  plot(c2pl1_fi(i,1),c2pl1_fi(i,2),'r*'), pause,
        end
        for i=1:length(c2pl_2),
            subplot(1,2,1),hold on, plot(pl2_phi(1,1),pl2_phi(1,2),'g*'), plot3(c2pl_2(i,1),c2pl_2(i,2),c2pl_2(i,3),'co'),
            subplot(1,2,2),hold on,  plot(c2pl2_fi(i,1),c2pl2_fi(i,2),'y*'), pause,
        end
    end

    % now calibrate as before
    % find the function that converts from image to global coordinates
    c1_beta_im2gl=cal_polyn_im2gl(c1pl_1,c1pl_2,c1pl1_fi,c1pl2_fi,c1pixnum,LSm_c,poly_n);
    %find the function that converts from global to image coordinates
    c1_beta_gl2im=cal_polyn_gl2im(c1pl_1,c1pl_2,c1pl1_fi,c1pl2_fi,c1pixnum,LSm_c,poly_n);




    if wanna_noRIC==1
        %combine image coordinates into single sets of points
        c1im_comb=[c1pl1_fi;c1pl2_fi];
        %use calibration function to convert from image coordinates to global coordinates
        x=c1im_comb(:,1); y=c1im_comb(:,2); a=c1pixnum(2)/2; b=c1pixnum(1)/2;
        [c1_pl1_ud(:,1)]=polynmapping(x,y,a,b,c1_beta_im2gl(:,1),poly_n);
        [c1_pl1_ud(:,2)]=polynmapping(x,y,a,b,c1_beta_im2gl(:,2),poly_n);
        [c1_pl2_ud(:,1)]=polynmapping(x,y,a,b,c1_beta_im2gl(:,3),poly_n);
        [c1_pl2_ud(:,2)]=polynmapping(x,y,a,b,c1_beta_im2gl(:,4),poly_n);

        %perform a refractive index correction on these points
        [c1_pl1_ud,c1_pl2_ud]=ncor_new(c1_pl1_ud,c1_pl2_ud,pl_1_o,pl_2_o);

        %recalculate the calibration functions using the corrected coordinates
        % find the function that converts from image to global coordinates
        c1_beta_im2gl_ud=cal_polyn_im2gl(c1_pl1_ud,c1_pl2_ud,c1im_comb,c1im_comb,c1pixnum,LSm_c,poly_n);
        %find the function that converts from global to image coordinates
        c1_beta_gl2im_ud=cal_polyn_gl2im(c1_pl1_ud,c1_pl2_ud,c1im_comb,c1im_comb,c1pixnum,LSm_c,poly_n);

        if wanna_overwrite==1
            c1_beta_im2gl=c1_beta_im2gl_ud;
            c1_beta_gl2im=c1_beta_gl2im_ud;
        end
    end





    % save these for use in matlab
    cd(save_dir)
    save(['polycal_mat_',num2str(current_ref)],'c1_beta_im2gl','c1_beta_gl2im')
    cd(dir)
    % due to accuracy constraints of intel fortran 77 on single precision limit number of sig figs
    c1_beta_im2gl_orig=c1_beta_im2gl; %save orig versions
    c1_beta_im2gl(abs(c1_beta_im2gl)<fort_lim)=0;

    % Now test the calibration by mapping the image coordinates of the calibration points back into the global domain,
    % and also calculating the residuals (and then plot these)

    % camera 1 plane 1
    x=c1pl1_fi(:,1); y=c1pl1_fi(:,2); a=c1pixnum(2)/2; b=c1pixnum(1)/2;
    [c1F_test_x_pl1]=polynmapping(x,y,a,b,c1_beta_im2gl(:,1),poly_n); [c1F_test_x_pl1_orig]=polynmapping(x,y,a,b,c1_beta_im2gl_orig(:,1),poly_n);
    [c1F_test_y_pl1]=polynmapping(x,y,a,b,c1_beta_im2gl(:,2),poly_n); [c1F_test_y_pl1_orig]=polynmapping(x,y,a,b,c1_beta_im2gl_orig(:,2),poly_n);
    % camera 1 plane 2
    x=c1pl2_fi(:,1); y=c1pl2_fi(:,2); a=c1pixnum(2)/2; b=c1pixnum(1)/2;
    [c1F_test_x_pl2]=polynmapping(x,y,a,b,c1_beta_im2gl(:,3),poly_n); [c1F_test_x_pl2_orig]=polynmapping(x,y,a,b,c1_beta_im2gl_orig(:,3),poly_n);
    [c1F_test_y_pl2]=polynmapping(x,y,a,b,c1_beta_im2gl(:,4),poly_n); [c1F_test_y_pl2_orig]=polynmapping(x,y,a,b,c1_beta_im2gl_orig(:,4),poly_n);

    %test by mapping gamma coords back into global domain, and see if they match

    % camera 1
    x=c1cp_fi(1:3,1); y=c1cp_fi(1:3,2); a=c1pixnum(2)/2; b=c1pixnum(1)/2;
    [c1_cp_pl1(:,1)]=polynmapping(x,y,a,b,c1_beta_im2gl(:,1),poly_n); [c1_cp_pl1(:,2)]=polynmapping(x,y,a,b,c1_beta_im2gl(:,2),poly_n);
    x=c1cp_fi(4:6,1); y=c1cp_fi(4:6,2); a=c1pixnum(2)/2; b=c1pixnum(1)/2;
    [c1_cp_pl2(:,1)]=polynmapping(x,y,a,b,c1_beta_im2gl(:,3),poly_n); [c1_cp_pl2(:,2)]=polynmapping(x,y,a,b,c1_beta_im2gl(:,4),poly_n);

    if wanna_remap==1
        figure(25), clf, hold on
        plot3(c1_cp_pl1(:,1),c1_cp_pl1(:,2),(ones(size(c1_cp_pl1(:,1)))*pl_1_o(3)),'go')
        plot3(c1_cp_pl2(:,1),c1_cp_pl2(:,2),(ones(size(c1_cp_pl2(:,1)))*pl_2_o(3)),'go')
        axis equal, grid on
    end



    if wanna_remap==1
        figure(3),clf,
        plot3(c1F_test_x_pl1,c1F_test_y_pl1,(ones(size(c1F_test_x_pl1))*pl_1_o(3)),'ro'), hold on, % plot c1pl1 - from cal
        plot3(c1F_test_x_pl2,c1F_test_y_pl2,(ones(size(c1F_test_x_pl2))*pl_2_o(3)),'ro'),                          % plot c1pl2 - from cal

        %plot3(pl_1(:,1),pl_1(:,2),pl_1(:,3),'k*'),hold on, plot3(pl_2(:,1),pl_2(:,2),pl_2(:,3),'k*'), axis equal, grid on, %plot c1pl1 and pl2 ideal

        plot3(c1pl_1(:,1),c1pl_1(:,2),(ones(size(c1pl_1,1))*pl_1_o(3)),'k*'),hold on, plot3(c1pl_2(:,1),c1pl_2(:,2),(ones(size(c1pl_2,1))*pl_2_o(3)),'k*') %plot c1pl1 and pl2 ideal
        axis equal, grid on,

        title('Test 1, mapping existing calibration points back into world coords');
        xlabel('x'), ylabel('y'), zlabel('z'), set(gca,'ZDir','reverse')
    end


    % single point check - plot find a single point on each image, then map this
    % point back into global space on the front and back planes, and check that lines of sight converge where they should.
    single_pointcheck=1;
    if single_pointcheck==1
        pn=1;  %identify single point, one of the gamma points will do, choose 1-6
        % camera 1 plane 1
        xtra_x=0; xtra_y=0;
        x=c1cp_fi(pn,1)+xtra_x; y=c1cp_fi(pn,2)+xtra_y; a=c1pixnum(2)/2; b=c1pixnum(1)/2;
        [sing_c1x_pl1]=polynmapping(x,y,a,b,c1_beta_im2gl(:,1),poly_n); [sing_c1y_pl1]=polynmapping(x,y,a,b,c1_beta_im2gl(:,2),poly_n);
        % camera 1 plane 2
        [sing_c1x_pl2]=polynmapping(x,y,a,b,c1_beta_im2gl(:,3),poly_n); [sing_c1y_pl2]=polynmapping(x,y,a,b,c1_beta_im2gl(:,4),poly_n);

    end
    figure(3)
    plot3([sing_c1x_pl1,sing_c1x_pl2],[sing_c1y_pl1,sing_c1y_pl2],[pl_1_o(3),pl_2_o(3)],'m')



    %-------------------------------------------recommend a maximum volume size
    c1pl1lox=min(c1pl_1(:,1)); c1pl1hix=max(c1pl_1(:,1)); c1pl1loy=min(c1pl_1(:,2)); c1pl1hiy=max(c1pl_1(:,2));
    c1pl2lox=min(c1pl_2(:,1)); c1pl2hix=max(c1pl_2(:,1)); c1pl2loy=min(c1pl_2(:,2)); c1pl2hiy=max(c1pl_2(:,2));
    plot3([c1pl1lox,c1pl1lox,c1pl1hix,c1pl1hix,c1pl1lox],[c1pl1loy,c1pl1hiy,c1pl1hiy,c1pl1loy,c1pl1loy],ones(1,5)*pl_1_o(3),'r:')
    plot3([c1pl2lox,c1pl2lox,c1pl2hix,c1pl2hix,c1pl2lox],[c1pl2loy,c1pl2hiy,c1pl2hiy,c1pl2loy,c1pl2loy],ones(1,5)*pl_2_o(3),'r:')

    lox=max([c1pl1lox,c1pl2lox]); hix=min([c1pl1hix,c1pl2hix]);
    loy=max([c1pl1loy,c1pl2loy]); hiy=min([c1pl1hiy,c1pl2hiy]);

    disp(' ')
    disp(['Maximum Volume Coords x=[',num2str(lox),',',num2str(hix),'], y=[',num2str(loy),',',num2str(hiy),']'])
    disp(' ')
    plot3([lox,lox,hix,hix,lox],[loy,hiy,hiy,loy,loy],ones(1,5)*pl_1_o(3),'k')
    plot3([lox,lox,hix,hix,lox],[loy,hiy,hiy,loy,loy],ones(1,5)*pl_2_o(3),'k')
    plot3([lox,lox],[loy,loy],[pl_1_o(3),pl_2_o(3)],'k'), plot3([hix,hix],[loy,loy],[pl_1_o(3),pl_2_o(3)],'k'), plot3([lox,lox],[hiy,hiy],[pl_1_o(3),pl_2_o(3)],'k'), plot3([hix,hix],[hiy,hiy],[pl_1_o(3),pl_2_o(3)],'k')

    %--------------------------------------------------------------------------

    % now remap vice versa, by taking the general global coordinates and converting into image coords,
    % then plotting these on each image to see if they are ok,
    % also can check the order of the points, to see if they correspond ok

    % camera 1 plane 1
    x=c1pl_1(:,1); y=c1pl_1(:,2); a=0; b=0;
    [c1F_test2_x_pl1]=polynmapping(x,y,a,b,c1_beta_gl2im(:,1),poly_n);
    [c1F_test2_y_pl1]=polynmapping(x,y,a,b,c1_beta_gl2im(:,2),poly_n);
    % camera 1 plane 2
    x=c1pl_2(:,1); y=c1pl_2(:,2); a=0; b=0;
    [c1F_test2_x_pl2]=polynmapping(x,y,a,b,c1_beta_gl2im(:,3),poly_n);
    [c1F_test2_y_pl2]=polynmapping(x,y,a,b,c1_beta_gl2im(:,4),poly_n);

    if wanna_remap2==1
        figure(4),clf,
        subplot(1,cams,1), imagesc(im1f), hold on,
        title('Test 2, mapping world coords back into image coords');colormap('gray')
        for i=1:length(c1F_test2_x_pl1)
            subplot(1,cams,1),plot(c1F_test2_x_pl1(i),c1F_test2_y_pl1(i),'rx'), text(c1F_test2_x_pl1(i),c1F_test2_y_pl1(i),num2str(i),'color','r')
        end
        for i=1:length(c1F_test2_x_pl2)
            subplot(1,cams,1),plot(c1F_test2_x_pl2(i),c1F_test2_y_pl2(i),'gx'), text(c1F_test2_x_pl2(i),c1F_test2_y_pl2(i),num2str(i),'color','g')
        end
    end

    if wanna_save_thesis_im==1; figure(11), colormap('gray'), imagesc(im1); hold on,
        plot(c1_cp(1:3,1),c1_cp(1:3,2),'go','linewidth',2); plot(c1_cp(4:6,1),c1_cp(4:6,2),'ro','linewidth',2)
        set(gca,'xtick',[],'ytick',[])
        plot(c1F_test2_x_pl1,c1F_test2_y_pl1,'rx','linewidth',2), plot(c1F_test2_x_pl2,c1F_test2_y_pl2,'gx','linewidth',2)
        daspect([1 1 1])
        thesis_imagesave('cal_points_all',3)
        figure(1)
    end




    if wanna_remap3==1
        figure(7), clf, hold on, plot(c1F_test2_x_pl1,c1F_test2_y_pl1,'rx'), plot(c1F_test2_x_pl2,c1F_test2_y_pl2,'gx')
        figure(2),
        subplot(1,2,1),hold on, plot(c1F_test2_x_pl1,c1F_test2_y_pl1,'rx')
        subplot(1,2,2),hold on, plot(c1F_test2_x_pl2,c1F_test2_y_pl2,'gx')
    end


    % new test - extend the original global coords way outside the calibration
    % range and so they cover the entire image domain, to check the calibration
    % in the case that the calibration plate is smaller than the image
    wanna_remap4=0;
    if wanna_remap4==1
        % camera 1 plane 1

        c1=0;
        for xpos=-20:20
            for ypos=-20:20
                c1=c1+1;
                pl_1_new(c1,1)=(xpos-pl_1_cen(1))*pl_1_d(1);
                pl_1_new(c1,2)=(ypos-pl_1_cen(2))*pl_1_d(1);
                pl_1_new(c1,3)=pl_1_o(3);
            end
        end

        pl_2_new=[pl_1_new(:,1) pl_1_new(:,2) ones(size(pl_1_new(:,1)))*pl_2_o(3)];

        x=pl_1_new(:,1); y=pl_1_new(:,2); a=0; b=0;
        [TESTx_pl1]=polynmapping(x,y,a,b,c1_beta_gl2im(:,1),poly_n);
        [TESTy_pl1]=polynmapping(x,y,a,b,c1_beta_gl2im(:,2),poly_n);
        % camera 1 plane 2
        x=pl_2_new(:,1); y=pl_2_new(:,2); a=0; b=0;
        [TESTx_pl2]=polynmapping(x,y,a,b,c1_beta_gl2im(:,3),poly_n);
        [TESTy_pl2]=polynmapping(x,y,a,b,c1_beta_gl2im(:,4),poly_n);

        % create artificial offset
        %     for a=1:size(TESTx_pl1,1)
        %        if TESTx_pl1(a)>600
        %            TESTx_pl1(a)=TESTx_pl1(a)+12;
        %        end
        %     end
        %     for a=1:size(TESTx_pl2,1)
        %        if TESTx_pl2(a)>600
        %            TESTx_pl2(a)=TESTx_pl2(a)+12;
        %        end
        %     end

        figure(2),
        subplot(1,2,1),hold on, plot(TESTx_pl1,TESTy_pl1,'ro')
        subplot(1,2,2),hold on, plot(TESTx_pl2,TESTy_pl2,'go')



        %     THIS ATTEMPTS TO CALIBRATE RIC CHANGE - DOES NOT WORK
        %
        %     % create a new calibration with these points and global positions
        %     % find the function that converts from image to global coordinates
        %     poly_n=3;
        %     c1_beta_im2gl_NEW=cal_polyn_im2gl(pl_1_new,pl_2_new,[TESTx_pl1,TESTy_pl1],[TESTx_pl2,TESTy_pl2],c1pixnum,LSm_c,poly_n);
        %     %find the function that converts from global to image coordinates
        %     c1_beta_gl2im_NEW=cal_polyn_gl2im(pl_1_new,pl_2_new,[TESTx_pl1,TESTy_pl1],[TESTx_pl2,TESTy_pl2],c1pixnum,LSm_c,poly_n);
        %
        %     % now use this new calibration to transform the global points and plot
        %     x=pl_1_new(:,1); y=pl_1_new(:,2); a=0; b=0;
        %     [TESTx_pl1]=polynmapping(x,y,a,b,c1_beta_gl2im_NEW(:,1),poly_n);
        %     [TESTy_pl1]=polynmapping(x,y,a,b,c1_beta_gl2im_NEW(:,2),poly_n);
        %     % camera 1 plane 2
        %     x=pl_2_new(:,1); y=pl_2_new(:,2); a=0; b=0;
        %     [TESTx_pl2]=polynmapping(x,y,a,b,c1_beta_gl2im_NEW(:,3),poly_n);
        %     [TESTy_pl2]=polynmapping(x,y,a,b,c1_beta_gl2im_NEW(:,4),poly_n);
        %
        %     figure(2),
        %     subplot(1,2,1),hold on, plot(TESTx_pl1,TESTy_pl1,'r*')
        %     subplot(1,2,2),hold on, plot(TESTx_pl2,TESTy_pl2,'g*')

    end







    %find residuals - difference between measured (pl_1...) and reconstructed from fit (Fnew_x1....)

    %residuals between plane coords and plane coords remapped from images -
    %with truncation if applied
    c1residuals=[c1pl_1(:,1)-c1F_test_x_pl1;c1pl_1(:,2)-c1F_test_y_pl1;c1pl_2(:,1)-c1F_test_x_pl2;c1pl_2(:,2)-c1F_test_y_pl2];
    % same as above but without truncation if applied
    c1residuals_orig=[c1pl_1(:,1)-c1F_test_x_pl1_orig;c1pl_1(:,2)-c1F_test_y_pl1_orig;c1pl_2(:,1)-c1F_test_x_pl2_orig;c1pl_2(:,2)-c1F_test_y_pl2_orig];
    %calculating the mean and max numbers for above distributions
    c1mr=mean(abs(c1residuals)); c1maxr=max(abs(c1residuals)); c1mr_orig=mean(abs(c1residuals_orig)); c1maxr_orig=max(abs(c1residuals_orig));
    %calculting residuals between global points mapped back into image
    %coordinates using gl2im transformation
    c1residuals_pix=[c1pl1_fi(:,1)-c1F_test2_x_pl1 ; c1pl1_fi(:,2)-c1F_test2_y_pl1 ; c1pl2_fi(:,1)-c1F_test2_x_pl2 ; c1pl2_fi(:,2)-c1F_test2_y_pl2];
    %calculating the mean and max numbers for above distributions
    c1mr_pix=mean(abs(c1residuals_pix)); c1maxr_pix=max(abs(c1residuals_pix));

    if wanna_error==1
        figure(6), title('Polynominal Calibration Error Distribution')
        subplot(2,1,1), hist(abs(c1residuals)); xlabel('error (mm)'), ylabel('number of calibration points'); title('cam 1: im to gl')
        subplot(2,1,2), hist(abs(c1residuals_pix)); xlabel('error (pixels)'), ylabel('number of calibration points');  title('cam 1: gl to im')
    end

    disp('When converting from image to global coords')
    disp(['The mean error for cam 1 is ',num2str(c1mr),' mm, with a maximum of ',num2str(c1maxr),' mm'])
    disp('')
    disp('When converting from global to image coords')
    disp(['The mean error for cam 1 is ',num2str(c1mr_pix),' pixels, with a maximum of ',num2str(c1maxr_pix),' pixels'])

    if wanna_output==1;
        disp(' ')
        disp(' ')
        disp(' ')
        disp(' ')
        disp(['       c',num2str(output_ref),'be(1,1)=',num2str(c1_beta_im2gl(1,1)),'; c',num2str(output_ref),'be(1,2)=',num2str(c1_beta_im2gl(1,2))])
        disp(['       c',num2str(output_ref),'be(1,3)=',num2str(c1_beta_im2gl(1,3)),'; c',num2str(output_ref),'be(1,4)=',num2str(c1_beta_im2gl(1,4))])
        disp(['       c',num2str(output_ref),'be(2,1)=',num2str(c1_beta_im2gl(2,1)),'; c',num2str(output_ref),'be(2,2)=',num2str(c1_beta_im2gl(2,2))])
        disp(['       c',num2str(output_ref),'be(2,3)=',num2str(c1_beta_im2gl(2,3)),'; c',num2str(output_ref),'be(2,4)=',num2str(c1_beta_im2gl(2,4))])
        disp(['       c',num2str(output_ref),'be(3,1)=',num2str(c1_beta_im2gl(3,1)),'; c',num2str(output_ref),'be(3,2)=',num2str(c1_beta_im2gl(3,2))])
        disp(['       c',num2str(output_ref),'be(3,3)=',num2str(c1_beta_im2gl(3,3)),'; c',num2str(output_ref),'be(3,4)=',num2str(c1_beta_im2gl(3,4))])
        disp(['       c',num2str(output_ref),'be(4,1)=',num2str(c1_beta_im2gl(4,1)),'; c',num2str(output_ref),'be(4,2)=',num2str(c1_beta_im2gl(4,2))])
        disp(['       c',num2str(output_ref),'be(4,3)=',num2str(c1_beta_im2gl(4,3)),'; c',num2str(output_ref),'be(4,4)=',num2str(c1_beta_im2gl(4,4))])
        disp(['       c',num2str(output_ref),'be(5,1)=',num2str(c1_beta_im2gl(5,1)),'; c',num2str(output_ref),'be(5,2)=',num2str(c1_beta_im2gl(5,2))])
        disp(['       c',num2str(output_ref),'be(5,3)=',num2str(c1_beta_im2gl(5,3)),'; c',num2str(output_ref),'be(5,4)=',num2str(c1_beta_im2gl(5,4))])
        disp(['       c',num2str(output_ref),'be(6,1)=',num2str(c1_beta_im2gl(6,1)),'; c',num2str(output_ref),'be(6,2)=',num2str(c1_beta_im2gl(6,2))])
        disp(['       c',num2str(output_ref),'be(6,3)=',num2str(c1_beta_im2gl(6,3)),'; c',num2str(output_ref),'be(6,4)=',num2str(c1_beta_im2gl(6,4))])
        disp(['       c',num2str(output_ref),'be(7,1)=',num2str(c1_beta_im2gl(7,1)),'; c',num2str(output_ref),'be(7,2)=',num2str(c1_beta_im2gl(7,2))])
        disp(['       c',num2str(output_ref),'be(7,3)=',num2str(c1_beta_im2gl(7,3)),'; c',num2str(output_ref),'be(7,4)=',num2str(c1_beta_im2gl(7,4))])
        disp(['       c',num2str(output_ref),'be(8,1)=',num2str(c1_beta_im2gl(8,1)),'; c',num2str(output_ref),'be(8,2)=',num2str(c1_beta_im2gl(8,2))])
        disp(['       c',num2str(output_ref),'be(8,3)=',num2str(c1_beta_im2gl(8,3)),'; c',num2str(output_ref),'be(8,4)=',num2str(c1_beta_im2gl(8,4))])
        disp(['       c',num2str(output_ref),'be(9,1)=',num2str(c1_beta_im2gl(9,1)),'; c',num2str(output_ref),'be(9,2)=',num2str(c1_beta_im2gl(9,2))])
        disp(['       c',num2str(output_ref),'be(9,3)=',num2str(c1_beta_im2gl(9,3)),'; c',num2str(output_ref),'be(9,4)=',num2str(c1_beta_im2gl(9,4))])
        disp(['       c',num2str(output_ref),'be(10,1)=',num2str(c1_beta_im2gl(10,1)),'; c',num2str(output_ref),'be(10,2)=',num2str(c1_beta_im2gl(10,2))])
        disp(['       c',num2str(output_ref),'be(10,3)=',num2str(c1_beta_im2gl(10,3)),'; c',num2str(output_ref),'be(10,4)=',num2str(c1_beta_im2gl(10,4))])
        disp(' ')
        disp(['       c1u0=',num2str(c1pixnum(1)/2),'; c1v0=',num2str(c1pixnum(2)/2),])
        disp(' ')
        disp(['       pl_1_o(1)=',num2str(pl_1_o(1)),'; pl_1_o(2)=',num2str(pl_1_o(2))])
        disp(['       pl_1_o(3)=',num2str(pl_1_o(3)),'; pl_2_o(1)=',num2str(pl_2_o(1))])
        disp(['       pl_2_o(2)=',num2str(pl_2_o(2)),'; pl_2_o(3)=',num2str(pl_2_o(3))])
        disp(' ')
        disp(' ')
        disp(' ')
    elseif wanna_output==2
        combined_save_mat=[c1_beta_im2gl;pl_1_o ,c1pixnum(1)/2;pl_2_o ,c1pixnum(2)/2];
        disp(['Saving calibration data to file: ','cal_dat_',sprintf('%0.3i',output_ref),'.dat'])
        cd(save_dir)
        save(['cal_dat_',sprintf('%0.3i',output_ref),'.txt'],'combined_save_mat','-ASCII')
        cd(dir)
    end

end


%--------------------------------------------------------------------------------------------------
%----- ESTIMATES USING A SINGLE VERTICAL AND HORISONTAL LINE---------------------------------------
%--------------------------------------------------------------------------------------------------
function[pl1_fi,pl2_fi,dubpco,pl1_inr,pl2_inr,cp_fi]=point_locator(cp,im,pixnum,pl_1_num,pl_2_num,boxs,cam,dubpco,pl1_phi,pl2_phi,pl_1_d,pl_2_d,pl_1_o,pl_2_o,LSm_pf)
disp(cp)
pixnum=size(im);
% plot the estimates, on figure(1)
figure(1)
subplot(2,2,cam),hold on,
plot(cp(1:3,1),cp(1:3,2),'go')
plot(cp(4:6,1),cp(4:6,2),'ro')

% Now ensure you have identified the local peaks,
% next use these to estimate the centers of these calibration points in a Gaussian-esk fashion.
for a=1:6
    [cp_co(a,1:2),cp_fi(a,1:2),dubpco,in_range]=find_loc_peak2(cp(a,:),boxs,im,0,dubpco);
end

%plot the confirmed estimates and gaussian centers
subplot(2,2,cam)
plot(cp_co(1:3,1),cp_co(1:3,2),'bo')
plot(cp_co(4:6,1),cp_co(4:6,2),'mo')
plot(cp_fi(1:3,1),cp_fi(1:3,2),'bx')
plot(cp_fi(4:6,1),cp_fi(4:6,2),'mx')

% Ok maybe include some kind of check part here, to confirm these points are ok

% now calculate the equation of the lines in x and y using a least squares approach,
% and also the line pitch
% use these to define the area in which the next points will be searched for
% do this iteratively a couple of times to update the estimation of the outer points is correct.

%calculate how many points around point 1 on the left and right for plane 1
if rem(pl_1_num(1),2)==0 %then even
    pl_1_left=(pl_1_num(1)/2)-1; pl_1_right=pl_1_left+1;
else %then its odd
    pl_1_left=floor(pl_1_num(1)/2); pl_1_right=pl_1_left;
end

%calculate how many points around point 1 on the left and right for plane 2
if rem(pl_2_num(1),2)==0 %then even
    pl_2_left=(pl_2_num(1)/2)-1; pl_2_right=pl_2_left+1;
else %then its odd
    pl_2_left=floor(pl_2_num(1)/2); pl_2_right=pl_2_left;
end

%----------------------------------PLANE 1-----------------------------------------------

figure(2),
% plane 1, line between points 1 and 2
x=[cp_fi(1,1),cp_fi(2,1)]'; y=[cp_fi(1,2),cp_fi(2,2)]';
X=[x, ones(size(x))]      ; beta1=X\y;
% plot this line then for x=1 and pixnum
x_line=[1;pixnum(2)];  y_line=(beta1(1)*x_line)+beta1(2);
subplot(2,2,cam),hold on, plot(x_line,y_line),
% plane 2, line between points 1 and 3
x=[cp_fi(1,1),cp_fi(3,1)]';  y=[cp_fi(1,2),cp_fi(3,2)]';
X=[y, ones(size(x))]      ;  beta2=X\x;
% plot this line then for x=1 and 100
y_line=[1;pixnum(1)];  x_line=(beta2(1)*y_line)+beta2(2);
subplot(2,2,cam), plot(x_line,y_line)

% horisontal pitch - x distance between points 1 and 2
h_pitch=cp_fi(2,1)-cp_fi(1,1);
%h_pitch2=cp_fi(3,1)-cp_fi(1,1); h_pitchall=abs([h_pitch,h_pitch2]);
%if h_pitchall(1)>h_pitchall(2); h_pitch=h_pitch,; else h_pitch=h_pitch2,; end
% vertical pitch - y distance between points 1 and 3
v_pitch=cp_fi(3,2)-cp_fi(1,2);

%predict the positions of all the points
count=0;
for xpos=-pl_1_left:pl_1_right
    for ypos=-pl_1_left:pl_1_right
        count=count+1;
        newx=(cp_fi(1,1)+(xpos*h_pitch));
        newy=(cp_fi(1,2)+(ypos*v_pitch));
        newymod=((beta1(1)*newx)+beta1(2));
        newxmod=((beta2(1)*newy)+beta2(2));
        pl1_est(count,1:2)=[(newxmod+(xpos*h_pitch)),(newymod+(ypos*v_pitch))];
    end
end
subplot(2,2,cam), plot(pl1_est(:,1),pl1_est(:,2),'bs')
%show first two points on figure 1 to ensure coordinate system is correct
figure(1),subplot(2,2,cam),hold on, plot(pl1_est(1:2,1),pl1_est(1:2,2),'g*') ,figure(2)

%----------------------------------PLANE 2-----------------------------------------------
% plane 2, line between points 1 and 2
x=[cp_fi(4,1),cp_fi(5,1)]'; y=[cp_fi(4,2),cp_fi(5,2)]';
X=[x, ones(size(x))]      ; beta3=X\y;
% plot this line then for x=1 and pixnum
x_line=[1;pixnum(2)];  y_line=(beta3(1)*x_line)+beta3(2);
subplot(2,2,cam), plot(x_line,y_line,'m')
% plane 2, line between points 1 and 3
x=[cp_fi(4,1),cp_fi(6,1)]';  y=[cp_fi(4,2),cp_fi(6,2)]';
X=[y, ones(size(x))]      ;  beta4=X\x;
% plot this line then for x=1 and 100
y_line=[1;pixnum(1)];  x_line=(beta4(1)*y_line)+beta4(2);
subplot(2,2,cam), plot(x_line,y_line,'m')

% horisontal pitch - x distance between points 1 and 2
h_pitch=cp_fi(5,1)-cp_fi(4,1);
% vertical pitch - y distance between points 1 and 3
v_pitch=cp_fi(6,2)-cp_fi(4,2);

%predict the positions of all the points
count=0;
for xpos=-pl_2_left:pl_2_right
    for ypos=-pl_2_left:pl_2_right
        count=count+1;
        newx=(cp_fi(4,1)+(xpos*h_pitch));
        newy=(cp_fi(4,2)+(ypos*v_pitch));
        newymod=((beta3(1)*newx)+beta3(2));
        newxmod=((beta4(1)*newy)+beta4(2));
        pl2_est(count,1:2)=[(newxmod+(xpos*h_pitch)),(newymod+(ypos*v_pitch))];
    end
end
subplot(2,2,cam), plot(pl2_est(:,1),pl2_est(:,2),'ms')
%show first two points on figure 1 to ensure coordinate system is correct
figure(1),subplot(2,2,cam),hold on, plot(pl2_est(1:2,1),pl2_est(1:2,2),'r*') ,figure(2)

% now we have the estimates of all the calibration points,
% perform peak confimation and gaussian peak estimation on all of these
for a=1:size(pl1_est,1)
    [pl1_co(a,1:2),pl1_fi(a,1:2),dubpco,pl1_inr(a)]=find_loc_peak2(pl1_est(a,1:2),boxs,im,1,dubpco);
end
for a=1:size(pl2_est,1)

    [pl2_co(a,1:2),pl2_fi(a,1:2),dubpco,pl2_inr(a)]=find_loc_peak2(pl2_est(a,1:2),boxs,im,1,dubpco);
end
subplot(2,2,cam)
plot(pl1_co(:,1),pl1_co(:,2),'bo')
plot(pl2_co(:,1),pl2_co(:,2),'mo')
plot(pl1_fi(:,1),pl1_fi(:,2),'bx')
plot(pl2_fi(:,1),pl2_fi(:,2),'mx')



%--------------------------------------------------------------------------------------------
% POINT ESTIMATION USING FULL LEAST SQUARES APPROACH ----------------------------------------
%--------------------------------------------------------------------------------------------
function[pl1_fi,pl2_fi,dubpco,pl_1,pl_2,cp_fi]=point_locator2(cp,imf,imb,pixnum,pl_num,boxs,cam,dubpco,pl1_phi,pl2_phi,pl_1_d,pl_2_d,pl_1_o,pl_2_o,LSm_pf,correct_method,pl2_4pts,yvar,xvar,pl_1_x,pl_1_y,pl_2_x,pl_2_y,pl_1_cen,pl_2_cen,confirm_coord_matching,poly_n)


pixnum=size(imf);
% plot the estimates, on figure(1)
figure(1)
subplot(1,2,(2*cam)-1),hold on,
plot(cp(1:3,1),cp(1:3,2),'go')
subplot(1,2,(2*cam)),hold on,
plot(cp(4:6,1),cp(4:6,2),'ro')

% Now ensure you have identified the local peaks,
% next use these to estimate the centers of these calibration points in a Gaussian-esk fashion.
for a=1:3
    [cp_co(a,1:2),cp_fi(a,1:2),dubpco,in_range]=find_loc_peak2(cp(a,:),boxs,imf,0,dubpco);
end
for a=4:6
    [cp_co(a,1:2),cp_fi(a,1:2),dubpco,in_range]=find_loc_peak2(cp(a,:),boxs,imb,0,dubpco);
end



%plot the confirmed estimates and gaussian centers
subplot(1,2,(2*cam)-1)
plot(cp_co(1:3,1),cp_co(1:3,2),'bo')
plot(cp_co(4:6,1),cp_co(4:6,2),'mo')
subplot(1,2,(2*cam))
plot(cp_fi(1:3,1),cp_fi(1:3,2),'bx')
plot(cp_fi(4:6,1),cp_fi(4:6,2),'mx')

% Ok maybe include some kind of check part here, to confirm these points are ok

% now calculate the equation of the lines in x and y using a least squares approach,
% and also the line pitch
% use these to define the area in which the next points will be searched for
% do this iteratively a couple of times to update the estimation of the outer points is correct.

%calculate pitches in x and y directions between points 1, 2 and 3 for plane 1
p1h_pitch12=cp_fi(2,1)-cp_fi(1,1);
p1v_pitch12=cp_fi(2,2)-cp_fi(1,2);
p1h_pitch13=cp_fi(3,1)-cp_fi(1,1);
p1v_pitch13=cp_fi(3,2)-cp_fi(1,2);
%calculate pitches in x and y directions between points 1, 2 and 3 for plane 2
p2h_pitch12=cp_fi(5,1)-cp_fi(4,1);
p2v_pitch12=cp_fi(5,2)-cp_fi(4,2);
p2h_pitch13=cp_fi(6,1)-cp_fi(4,1);
p2v_pitch13=cp_fi(6,2)-cp_fi(4,2);

%----------plot the global positions so you can check they are matching----
if confirm_coord_matching==1

    figure(8), clf, subplot(1,2,1), hold on
    for xpos=1:(pl_num(1))
        for ypos=1:(pl_num(2))
            plot(pl_1_x(ypos,xpos),pl_1_y(ypos,xpos),'rx')
            plot(pl_2_x(ypos,xpos),pl_2_y(ypos,xpos),'gx')
        end
    end
    plot(pl1_phi(:,1),pl1_phi(:,2),'ro')
    plot(pl2_phi(:,1),pl2_phi(:,2),'go')
    subplot(1,2,2), imagesc(im),colormap('gray'), hold on,
end

%--------------------------------------------------------------------------



%calculate first 25 inner points
count=0; counter1=0; counter2=0; plot_counter=0;

for xpos=-2:2,
    for ypos=-2:2,  count=count+1;
        pl1newx=(cp_fi(1,1)+(xpos*p1h_pitch12))+(ypos*p1h_pitch13);
        pl1newy=(cp_fi(1,2)+(ypos*p1v_pitch13))+(xpos*p1v_pitch12);
        pl2newx=(cp_fi(4,1)+(xpos*p2h_pitch12))+(ypos*p2h_pitch13);
        pl2newy=(cp_fi(4,2)+(ypos*p2v_pitch13))+(xpos*p2v_pitch12);

        % save for plotting later
        plot_counter=plot_counter+1;
        pl1newx_plot(plot_counter)=pl1newx;
        pl1newy_plot(plot_counter)=pl1newy;
        pl2newx_plot(plot_counter)=pl2newx;
        pl2newy_plot(plot_counter)=pl2newy;


        pl1_est(count,1:2)=[pl1newx,pl1newy]; pl2_est(count,1:2)=[pl2newx,pl2newy];
        %estimate the fine position of these new points
        [pl1_est(count,1:2),pl1_est_fi(count,1:2),dubpco,in_range1]=find_loc_peak2(pl1_est(count,1:2),boxs,imf,0,dubpco);
        [pl2_est(count,1:2),pl2_est_fi(count,1:2),dubpco,in_range2]=find_loc_peak2(pl2_est(count,1:2),boxs,imb,0,dubpco);
        if in_range1==1 %take only in range values for plane 1
            counter1=counter1+1;
            pl1_est_fi_yes(counter1,1:2)=pl1_est_fi(count,1:2);
            pl_1_glpos(counter1,1:3)=[pl_1_x(-ypos+pl_1_cen(2),xpos+pl_1_cen(1)),pl_1_y(-ypos+pl_1_cen(2),xpos+pl_1_cen(1)),pl_1_o(3)];
        end
        if in_range2==1 %take only in range values for plane 2
            counter2=counter2+1;
            pl2_est_fi_yes(counter2,1:2)=pl2_est_fi(count,1:2);
            pl_2_glpos(counter2,1:3)=[pl_2_x(-ypos+pl_2_cen(2),xpos+pl_2_cen(1)),pl_2_y(-ypos+pl_2_cen(2),xpos+pl_2_cen(1)),pl_2_o(3)];
        end

        if confirm_coord_matching==1 %plot a representation of both the calibration plate in global coords, and in image coords to show which points are being mapped with which other points
            figure(8), subplot(1,2,1),
            plot(pl_1_x(-ypos+pl_1_cen(2),xpos+pl_1_cen(1)),pl_1_y(-ypos+pl_1_cen(2),xpos+pl_1_cen(1)),'b*')
            plot(pl_2_x(-ypos+pl_2_cen(2),xpos+pl_2_cen(1)),pl_2_y(-ypos+pl_2_cen(2),xpos+pl_2_cen(1)),'m*')
            subplot(1,2,2), hold on,
            plot(pl1_est_fi_yes(counter1,1),pl1_est_fi_yes(counter1,2),'b*')
            plot(pl2_est_fi_yes(counter2,1),pl2_est_fi_yes(counter2,2),'m*')
            %return the plot handle back to figure(2)
            figure(2), subplot(1,2,cam), hold on
            pause
        end

    end
end



% plot the points as found
figure(2), subplot(1,2,(2*cam)-1), hold on
plot(pl1newx_plot,pl1newy_plot,'b^')
plot(pl1_est_fi_yes(1:counter1,1),pl1_est_fi_yes(1:counter1,2),'b*') %plot these

figure(2), subplot(1,2,(2*cam)), hold on
plot(pl2newx_plot,pl2newy_plot,'m^'),
plot(pl2_est_fi_yes(1:counter2,1),pl2_est_fi_yes(1:counter2,2),'m*') %plot these


%calculate mapping for glob to im coords
bet1=cal_polyn_gl2im(pl_1_glpos,pl_2_glpos,pl1_est_fi_yes,pl2_est_fi_yes,pixnum,LSm_pf,poly_n);

% now repeat the guess for the 25 points, and cyclically extend these outwards
% to take into account more and more points, and update these pitch estimates during each iteration
num_from_cen=1;  pl1Lco=1; pl1Rco=1; pl2Lco=1; pl2Rco=1; pl1Uco=1; pl1Dco=1; pl2Uco=1; pl2Dco=1;

for times=2:floor(max(pl_num)/2)
    num_from_cen=num_from_cen+1;
    %calculate points left and right of center to include in estimate (increases iteratively)
    if floor(pl_num(1)/2)>=num_from_cen; pl1Lco=pl1Lco+1; pl1Rco=pl1Rco+1; pl2Lco=pl2Lco+1; pl2Rco=pl2Rco+1; end

    if floor(pl_num(2)/2)>=num_from_cen; pl1Uco=pl1Uco+1; pl1Dco=pl1Dco+1; pl2Uco=pl2Uco+1; pl2Dco=pl2Dco+1; end

    %if floor(pl_num(1)/2)>=num_from_cen; pl1Lco=pl1Lco+1; pl1Rco=pl1Rco+1; pl2Lco=pl2Lco+1; pl2Rco=pl2Rco+1; end


    %find position of global coords
    count=0;
    for xpos=-pl1Lco:pl1Rco, for ypos=-pl1Uco:pl1Dco, count=count+1;
            pl_1_glpos_est(count,1:3)=[pl_1_x(-ypos+pl_1_cen(2),xpos+pl_1_cen(1)),pl_1_y(-ypos+pl_1_cen(2),xpos+pl_1_cen(1)),pl_1_o(3)]; %global position of current point
        end, end
    count=0;
    for xpos=-pl2Lco:pl2Rco, for ypos=-pl2Uco:pl2Dco, count=count+1;
            pl_2_glpos_est(count,1:3)=[pl_2_x(-ypos+pl_2_cen(2),xpos+pl_2_cen(1)),pl_2_y(-ypos+pl_2_cen(2),xpos+pl_2_cen(1)),pl_2_o(3)]; %global position of current point
        end, end
    % map the global positions back into the image coords for plane 1
    x=pl_1_glpos_est(:,1); y=pl_1_glpos_est(:,2); a=0; b=0;
    [pl1newx]=polynmapping(x,y,a,b,bet1(:,1),poly_n);
    [pl1newy]=polynmapping(x,y,a,b,bet1(:,2),poly_n);
    % map the global positions back into the image coords for plane 1
    x=pl_2_glpos_est(:,1); y=pl_2_glpos_est(:,2); a=0; b=0;
    [pl2newx]=polynmapping(x,y,a,b,bet1(:,3),poly_n);
    [pl2newy]=polynmapping(x,y,a,b,bet1(:,4),poly_n);

    figure(2), subplot(1,2,(2*cam)-1), hold on
    count=0; counter=0;
    % PLANE 1 cycle through each estimated image coord, locate the center to give a fine estimate, and ensure this is in range
    for xpos=-pl1Lco:pl1Rco,
        for ypos=-pl1Uco:pl1Dco,
            count=count+1;
            plot(pl1newx(count),pl1newy(count),'b^') %plot check for estimate of center
            pl1_est(count,1:2)=[pl1newx(count),pl1newy(count)]; %store in this form
            %estimate the fine position of these new points
            [pl1_est(count,1:2),pl1_est_fi(count,1:2),dubpco,in_range]=find_loc_peak2(pl1_est(count,1:2),boxs,imf,0,dubpco);
            if in_range==1 %take only in range values
                counter=counter+1;
                pl1_est_fi_yes(counter,1:2)=pl1_est_fi(count,1:2);
                plot(pl1_est_fi_yes(counter,1),pl1_est_fi_yes(counter,2),'b*') %plot these
                pl_1_glpos(counter,1:3)=[pl_1_x(-ypos+pl_1_cen(2),xpos+pl_1_cen(1)),pl_1_y(-ypos+pl_1_cen(2),xpos+pl_1_cen(1)),pl_1_o(3)];
            end
        end
    end

    figure(2), subplot(1,2,(2*cam)), hold on
    count=0; counter=0;
    % PLANE 2 cycle through each estimated image coord, locate the center to give a fine estimate, and ensure this is in range
    for xpos=-pl2Lco:pl2Rco,
        for ypos=-pl2Uco:pl2Dco,
            count=count+1;
            plot(pl2newx(count),pl2newy(count),'m^') %plot check for estimate of center
            pl2_est(count,1:2)=[pl2newx(count),pl2newy(count)]; %store in this form
            %estimate the fine position of these new points
            [pl2_est(count,1:2),pl2_est_fi(count,1:2),dubpco,in_range]=find_loc_peak2(pl2_est(count,1:2),boxs,imb,0,dubpco);
            if in_range==1 %take only in range values
                counter=counter+1;
                pl2_est_fi_yes(counter,1:2)=pl2_est_fi(count,1:2);
                plot(pl2_est_fi_yes(counter,1),pl2_est_fi_yes(counter,2),'m*') %plot these
                pl_2_glpos(counter,1:3)=[pl_2_x(-ypos+pl_2_cen(2),xpos+pl_2_cen(1)),pl_2_y(-ypos+pl_2_cen(2),xpos+pl_2_cen(1)),pl_2_o(3)];

            end
        end
    end

    %use least squares method to re-calibrate for these points, then use this to estimate position of next load of points
    bet1=cal_polyn_gl2im(pl_1_glpos,pl_2_glpos,pl1_est_fi_yes,pl2_est_fi_yes,pixnum,LSm_pf,poly_n);
end

%convert to older names
pl1_fi=pl1_est_fi_yes;  pl2_fi=pl2_est_fi_yes;
pl_1=pl_1_glpos; pl_2=pl_2_glpos;
figure(2), subplot(1,2,(2*cam)-1), hold on
plot(pl1_est(:,1),pl1_est(:,2),'bs')
figure(2), subplot(1,2,(2*cam)), hold on
plot(pl2_est(:,1),pl2_est(:,2),'ms')




%     % test out grid fit to better extrapolate points:
%      test_gridfit=1;
%     if test_gridfit==1
%         close all
%
%
%
%         % calculate pitch
% %         dx_x=cp_fi(2,1)-cp_fi(1,1);
% %         dx_y=cp_fi(2,2)-cp_fi(1,2);
% %         dy_x=cp_fi(3,1)-cp_fi(1,1);
% %         dy_y=cp_fi(3,2)-cp_fi(1,2);
%
%
%        x=pl1_fi; y=pl1_fi;
%        count=0;
%        for a=1:pl_num(1)
%            for b=1:pl_num(2)
%                count=count+1;
%                xmat(a,b)=pl1_fi(count,1);
%                ymat(a,b)=pl1_fi(count,2);
%            end
%        end
%
%        % calculate refined 1st order changes
%        xrange=-pl2Lco:pl2Rco;
%        yrange=-pl2Uco:pl2Dco;
%       dx_x=mean(diff(xmat,1,1),1);
%       Pdx_x = polyfit(xrange,dx_x,1);
%       dx_y=mean(diff(xmat,1,2),2);
%       Pdx_y = polyfit(xrange,dx_y',1);
%       dy_x=mean(diff(ymat,1,1),1);
%       Pdy_x = polyfit(yrange,dy_x,1);
%       dy_y=mean(diff(ymat,1,2),2);
%       Pdy_y = polyfit(yrange,dy_y',1);
%
%
%       figure(2), clf,
%       plot(xrange,dx_x,'b*'), hold on
%       plot(xrange,polyval(Pdx_x,xrange),'b')
%       plot(xrange,dx_y,'r*'), hold on
%       plot(xrange,polyval(Pdx_y,xrange),'r')
%       plot(yrange,dy_x,'g*'), hold on
%       plot(yrange,polyval(Pdy_x,yrange),'g')
%       plot(yrange,dy_y,'k*'), hold on
%       plot(yrange,polyval(Pdy_y,yrange),'k')
%
%       %         plot(-length(dx_x):2*length(dx_x),polyval(Pdx_x,-length(dx_x):2*l
% %         ength(dx_x)),'b')
%
% %        %
% %         figure(2), clf
% %         dx_x=mean(diff(xmat,2,1),1);
% %         dx_y=mean(diff(xmat,2,2),2);
% %         dy_x=mean(diff(ymat,2,1),1);
% %         dy_y=mean(diff(ymat,2,2),2);
% %         Pdx_x = polyfit(1:length(dx_x),dx_x,1);
% %         Pdx_y = polyfit(1:length(dx_y),dx_y',1);
% %         Pdy_x = polyfit(1:length(dy_x),dy_x,1);
% %         Pdy_y = polyfit(1:length(dy_y),dy_y',1);
% %
% %
% %         plot(mean(diff(diff(xmat,1,1),1,1),1),'b*'), hold on
% %         plot(-length(dx_x):2*length(dx_x),polyval(Pdx_x,-length(dx_x):2*length(dx_x)),'b')
% %         plot(mean(diff(diff(xmat,1,2),1,2),2),'r*')
% %         plot(-length(dx_y):2*length(dx_y),polyval(Pdx_y,-length(dx_y):2*length(dx_y)),'r')
% %         plot(mean(diff(diff(ymat,1,1),1,1),1),'g*')
% %         plot(-length(dy_x):2*length(dy_x),polyval(Pdy_x,-length(dy_x):2*length(dy_x)),'g')
% %         plot(mean(diff(diff(ymat,1,2),1,2),2),'k*')
% %         plot(-length(dy_y):2*length(dy_y),polyval(Pdy_y,-length(dy_y):2*length(dy_y)),'k')
%
%
%
%          c=0;
%         for ypos=-10:10,
%             for xpos=-10:10,
%                 count=count+1;
%                 c=c+1;
%                 newx(c)=(cp_fi(1,1)+(xpos*  polyval(Pdx_x,xpos)  ))+(ypos* polyval(Pdx_y,ypos));
%                 newy(c)=(cp_fi(1,2)+(ypos* polyval(Pdy_y,ypos)))+(xpos*polyval(Pdy_x,xpos));
%             end
%         end
%
%          figure(1), clf,
%          imagesc(imf), hold on
%          plot(xmat,ymat,'r*')
%          plot(newx,newy,'ro')
%         plot(cp_fi(1,1),cp_fi(1,2),'g*')
%
%
%
%
%         return
%     end









%------------------------------------------------------------------------------------------------
%------------------------------------------------------------------------------------------------
%------------------------------------------------------------------------------------------------
function[peak_loc_co,peak_loc_fi,dubpco,in_range]=find_loc_peak(initial_guess,boxs,im,dubp,dubpco)

pixnum=size(im);
cp(1)=initial_guess(2);
cp(2)=initial_guess(1);
%imdub=double(im);
x1=floor(cp(1)-(boxs/2));
x2=ceil(cp(1)+(boxs/2));
y1=floor(cp(2)-(boxs/2));
y2=ceil(cp(2)+(boxs/2));


if x1>1 & y1>1 & x2<pixnum(2) & y2<pixnum(1) % if box around point is within range
    in_range=1; %note down as an in range position
    loc_area=im(x1:x2,y1:y2);
    maxval=max(max(loc_area));
    [xnew,ynew]=find(loc_area==maxval);
    xnew=xnew+x1-1; ynew=ynew+y1-1;
    peak_loc_co=[ynew,xnew]; %coarse peak location

    for p=1:size(peak_loc_co,1) % do for each peak (even if there are multiple peaks

        % Perform Gaussian peak estimation
        xarr(1:3)=[imb(xnew(p)-1,ynew(p)),im(xnew(p),ynew(p)),im(xnew(p)+1,ynew(p))];
        yarr(1:3)=[im(xnew(p),ynew(p)-1),im(xnew(p),ynew(p)),im(xnew(p),ynew(p)+1)];

        %eliminate zeros which would bugger up the logs and give complex values
        xarr(xarr<=0)=0.0000001;
        yarr(yarr<=0)=0.0000001;

        %insert if loop to eliminate influence from zero elements (ie linear interp for 2 points, simple max for 1)
        peak_loc_fi(p,1)=xnew(p)+((log(xarr(1))-log(xarr(3)))/(2*log(xarr(1))-4*log(xarr(2))+2*log(xarr(3))));
        peak_loc_fi(p,2)=ynew(p)+((log(yarr(1))-log(yarr(3)))/(2*log(yarr(1))-4*log(yarr(2))+2*log(yarr(3))));

    end

    if size(peak_loc_co,1)>1
        % if 2 maxima, then the gaussian peak estimation should account for this somehow
        %    peak_loc_co, peak_loc_fi
        peak_loc_co=peak_loc_co(1,:);
        peak_loc_fi=peak_loc_fi(1,:);
        if dubp==1
            dubpco=dubpco+1;
        end
    end

    temp=peak_loc_fi;
    peak_loc_fi(1)=temp(2);
    peak_loc_fi(2)=temp(1);
else
    peak_loc_co=[0,0];peak_loc_fi=[0,0];
    in_range=0; % if not in range return a zero to this value
end

%-----------------------------------------------------------------------------------------------
%-----------------------------------------------------------------------------------------------
%-----------------------------------------------------------------------------------------------

function[peak_loc_co,peak_loc_fi,dubpco,in_range]=find_loc_peak2(initial_guess,boxs,im,dubp,dubpco)

% this version uses the particle tracking type code to estimate centroid

pixnum=size(im);
cp(1)=initial_guess(2);
cp(2)=initial_guess(1);
%imdub=double(im);

x1=floor(cp(1)-(boxs/2));
x2=ceil(cp(1)+(boxs/2));
y1=floor(cp(2)-(boxs/2));
y2=ceil(cp(2)+(boxs/2));

if x1>1 & y1>1 & x2<pixnum(1) & y2<pixnum(2)
    loc_area=im(x1:x2,y1:y2);
    loc_inten_sum=sum(sum(loc_area));%calculate sum of intensities for window, and if zero use as another criterion to reject point from calibration

    if loc_inten_sum>0 % if box around point is within range
        in_range=1; %note down as an in range position

        maxval=max(max(loc_area));
        [xnew,ynew]=find(loc_area==maxval);
        xnew=xnew+x1-1; ynew=ynew+y1-1;
        peak_loc_co=[ynew,xnew]; %coarse peak location

        for p=1:size(peak_loc_co,1) % do for each peak (even if there are multiple peaks

            centout = cntrd(im,peak_loc_co,boxs);   % use the centroid program to find the position of the peak
            if size(centout,1)>0 %catch values that are not included
                peak_loc_fi=centout(:,1:2);
            else
                peak_loc_co=[0,0];peak_loc_fi=[0,0];
                in_range=0; % if not in range return a zero to this value
            end

        end

        if size(peak_loc_co,1)>1
            % if 2 maxima, then the gaussian peak estimation should account for this somehow
            %    peak_loc_co, peak_loc_fi
            peak_loc_co=peak_loc_co(1,:);
            peak_loc_fi=peak_loc_fi(1,:);
            if dubp==1
                dubpco=dubpco+1;
            end
        end

        %   temp=peak_loc_fi;
        %   peak_loc_fi(1)=temp(2);
        %   peak_loc_fi(2)=temp(1);

    else
        peak_loc_co=[0,0];peak_loc_fi=[0,0];
        in_range=0; % if not in range return a zero to this value
    end
else
    peak_loc_co=[0,0];peak_loc_fi=[0,0];
    in_range=0; % if not in range return a zero to this value
end

%-----------------------------------------------------------------------------------------------
%-----------------------------------------------------------------------------------------------
%--------------------------------------------------------------------------

function[pl1,pl2]=ncor(pl1,pl2,pl_1_o,pl_2_o)

%refractive index correction

%define refractive indicies of water and perspex
n1=1.333333;
n2=1.495;
nrat=n1/n2;

%calc orig angles, calculate correction and apply
ang2x=(atan((pl2(:,1)-pl1(:,1))./(pl_2_o(3)-pl_1_o(3))));
ang2y=(atan((pl2(:,2)-pl1(:,2))./(pl_2_o(3)-pl_1_o(3))));
ang1x=asin((sin(ang2x)./nrat));
ang1y=asin((sin(ang2y)./nrat));
% calc corrected coords from
pl2(:,1)=pl1(:,1)+(tan(ang1x).*(pl_2_o(3)-pl_1_o(3)));
pl2(:,2)=pl1(:,2)+(tan(ang1y).*(pl_2_o(3)-pl_1_o(3)));


function[pl1,pl2]=ncor_new(pl1,pl2,pl_1_o,pl_2_o)

% maybe when you calibrate with the refractive index, you are already
% including a angular dependence into the results. Therefore, when you
% eliminate the shift, there is still the angular dependence present.
% to test:: try calibrating with the un refracted points, and then apply
% the refractive index correction.


% original version maps all image points back to both front and back
% planes, before correcting the positions of the back plane.

% This new version should


% new refractive index correction

% there should be 3 types: 1. convert to original
%                          2. convert to plus 90 deg
%                          3. convert to minus 90 degrees


%define refractive indicies of quartz and air
      n1=1.000292; % air at STP
      n2=1.4544;%1.4544; % quartz at 600nm
      nrat=n2/n1;
      t_glass=5; % thickness of quartz in mm
      
%calc orig angles, calculate correction and apply

       ang1x=(atan((pl2(:,1)-pl1(:,1))./(pl_2_o(3)-pl_1_o(3))));
       %ang1x=(-(pi/2)-ang1x);
       
       ang1y=(atan((pl2(:,2)-pl1(:,2))./(pl_2_o(3)-pl_1_o(3))));
       ang1x=ang1x-0.0;
       ang1y=ang1y+0.03;
       
       mean(ang1x*180/pi)
       
       ang2x=asin((sin(ang1x)/nrat));
       ang2y=asin((sin(ang1y)/nrat));

       mean(ang2x*180/pi)
       
       % calc corrected coords from
       pl1(:,1)=pl1(:,1)+(((t_glass./(cos(ang2x))).*sin(ang1x-ang2x))./cos(ang1x));
       pl1(:,2)=pl1(:,2)+(((t_glass./(cos(ang2y))).*sin(ang1y-ang2y))./cos(ang1y));    
       pl2(:,1)=pl2(:,1)+(((t_glass./(cos(ang2x))).*sin(ang1x-ang2x))./cos(ang1x));
       pl2(:,2)=pl2(:,2)+(((t_glass./(cos(ang2y))).*sin(ang1y-ang2y))./cos(ang1y));     
       
       corrections=((t_glass./(cos(ang2x))).*sin(ang1x-ang2x))
       pl1(:,1)
       %mean_shift=mean(tan(ang1x).*t_glass)